import React, { FC } from "react";

import Button from "../../../components/Button";
import Modal from "../../../components/Modal";
import { BankDetails } from "../../../models/account";
import BankDetailsForm from "./BankDetailsForm";
import { BankDetailsContainer } from "./styles";

type BankDetailsModalProps = {
  title: string;
  bankDetails: BankDetails;
  onCopyFromProfile?: () => void;
  onSubmit: (bankDetails: BankDetails) => void;
  onCancel: () => void;
  errors?: Array<String>;
};

const BankDetailsModal: FC<BankDetailsModalProps> = ({
  title,
  bankDetails,
  onCopyFromProfile,
  onSubmit,
  onCancel,
  errors,
}) => {
  return (
    <Modal title={title} onClose={onCancel} width={"35%"}>
      <BankDetailsContainer>
        <BankDetailsForm
          bankDetails={bankDetails}
          errors={errors}
          onSubmit={onSubmit}
          buttons={(submitting, valid) => (
            <>
              {bankDetails.propertyId && (
                <Button onClick={onCopyFromProfile} disabled={submitting}>
                  Copy From Profile
                </Button>
              )}
              <Button type="submit" disabled={submitting || !valid}>
                Update
              </Button>
            </>
          )}
        />
      </BankDetailsContainer>
    </Modal>
  );
};

export default BankDetailsModal;
