import React, { FC, useCallback } from "react";

import { offer_api_rto_RtoStateEnum } from "../../../../../../globalTypes";
import Button from "../../../../components/Button";
import Modal from "../../../../components/Modal";
import { Rto, RTO_STATE_TITLES } from "../../../../models/offer";
import RentCollectionForm from "./RentCollectionForm";
import { RentCollectionStatus } from "./RentCollectionForm/styles";
import { RentCollectionContainer } from "./styles";

type RentCollectionModalProps = {
  title: string;
  rentData: Rto;
  onSubmit: (rentData: Rto) => void;
  onCancel: () => void;
  errors?: Array<String>;
};

const RentCollectionModal: FC<RentCollectionModalProps> = ({
  title,
  rentData,
  onSubmit,
  onCancel,
  errors,
}) => {
  const getStateTitle = useCallback((rentData: Rto): string => {
    if (
      rentData.state ===
        offer_api_rto_RtoStateEnum.RTO_STATE_WAITING_FOR_STANDING_ORDER_SETUP &&
      rentData.paymentSetup
    ) {
      return "Waiting for payment";
    } else {
      return RTO_STATE_TITLES.get(rentData.state) || "Unknown";
    }
  }, []);

  return (
    <Modal title={title} onClose={onCancel} width={"35%"}>
      <RentCollectionContainer>
        {rentData.id && (
          <RentCollectionStatus>{getStateTitle(rentData)}</RentCollectionStatus>
        )}
        <RentCollectionForm
          rentData={rentData}
          errors={errors}
          onSubmit={(formData) => {
            formData.propertyId = rentData.propertyId;
            onSubmit(formData);
          }}
          buttons={(rentData, submitting, valid) => (
            <Button
              type="submit"
              hidden={!!rentData.id}
              disabled={submitting || !valid}
              width={"100%"}
            >
              Submit
            </Button>
          )}
        />
      </RentCollectionContainer>
    </Modal>
  );
};

export default RentCollectionModal;
