import ApolloClient from "apollo-client";

import {
  UserUpdate,
  UserUpdateVariables,
} from "../graphql/mutations/__generated__/UserUpdate";
import USER_UPDATE from "../graphql/mutations/UserUpdate.graphql";
import {
  UserGetById,
  UserGetByIdVariables,
} from "../graphql/queries/__generated__/UserGetById";
import USER_GET_BY_ID from "../graphql/queries/UserGetById.graphql";
import { AccountDetails, BankDetails } from "../models/account";

export interface AccountService {
  getAccountById: (userId: string) => Promise<AccountDetails>;
  updateAccount: (accountDetails: AccountDetails) => Promise<string>;
}

export const createAccountService = (
  client: ApolloClient<any>
): AccountService => {
  return {
    getAccountById: async (userId: string): Promise<AccountDetails> => {
      const resp = await client.query<UserGetById, UserGetByIdVariables>({
        query: USER_GET_BY_ID,
        fetchPolicy: "no-cache",
        variables: { id: userId },
      });

      return {
        id: resp?.data?.admin?.User?.id || "",
        firstName: resp?.data?.admin?.User?.details?.first_name || "",
        secondName: resp?.data?.admin?.User?.details?.last_name || "",
        email: resp?.data?.admin?.User?.contact?.email_primary || "",
        phone: resp?.data?.admin?.User?.contact?.phone_primary || "",
        postcode: resp?.data?.admin?.User?.location?.postcode || "",
        bankDetails: {
          accountName: resp.data?.admin?.User?.bank_details?.account_name || "",
          personalReference:
            resp.data?.admin?.User?.bank_details?.personal_reference || "",
          accountNumber:
            resp.data?.admin?.User?.bank_details?.account_number || "",
          sortCode: resp.data?.admin?.User?.bank_details?.sort_code || "",
        } as BankDetails,
      } as AccountDetails;
    },

    updateAccount: async (accountDetails: AccountDetails): Promise<string> => {
      const updateUserResp = await client.query<
        UserUpdate,
        UserUpdateVariables
      >({
        query: USER_UPDATE,
        fetchPolicy: "no-cache",
        variables: {
          id: accountDetails.id,
          firstName: accountDetails.firstName,
          secondName: accountDetails.secondName,
          email: accountDetails.email,
          phone: accountDetails.phone,
          bankDetails: {
            account_name: accountDetails.bankDetails?.accountName,
            personal_reference: accountDetails.bankDetails?.personalReference,
            account_number: accountDetails.bankDetails?.accountNumber,
            sort_code: accountDetails.bankDetails?.sortCode,
          },
        },
      });

      return updateUserResp.data?.admin?.updateUser?.id || "";
    },
  };
};
