import ApolloClient from "apollo-client";

import {
  common_api_generic_query_QueryFilterOperatorEnum as OperatorEnum,
  invoice_api_InvoiceStatusEnum,
} from "../../../globalTypes";
import {
  InvoicesList,
  InvoicesListVariables,
} from "../graphql/queries/__generated__/InvoicesList";
import INVOICES_LIST from "../graphql/queries/InvoicesList.graphql";
import { Invoice } from "../models/invoice";

export interface InvoiceService {
  getStandingOrderInvoices: (userId: string) => Promise<Array<Invoice>>;
}

export const createInvoiceService = (
  client: ApolloClient<any>
): InvoiceService => {
  return {
    getStandingOrderInvoices: async (
      userId: string
    ): Promise<Array<Invoice>> => {
      const resp = await client.query<InvoicesList, InvoicesListVariables>({
        query: INVOICES_LIST,
        fetchPolicy: "no-cache",
        variables: {
          filter: [
            {
              fields: ["correlation_target"],
              operator: OperatorEnum.EQUALS,
              value: ["standing-order"],
            },
            {
              fields: ["status"],
              operator: OperatorEnum.EQUALS,
              value: [invoice_api_InvoiceStatusEnum.INVOICE_OPEN],
            },
          ],
          pagination: { page: 1, perPage: 1000 },
          sort: [{ field: "created", order: -1 }],
        },
      });

      return (
        resp.data?.admin?.allInvoices?.invoices?.map(
          (inv) =>
            ({
              id: inv.invoice_id,
              correlationTarget: inv.correlation_target,
              correlationId: inv.correlation_id,
              correlationType: inv.correlation_type,
              setup: inv.setup,
            } as Invoice)
        ) || []
      );
    },
  };
};
